# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shiba.accountingmanagement import AccountingManagement
from shiba.shibaexceptions import ShibaParameterError

from datetime import date


def test_get_operations(connection):
    """get_operations routine test, with date object as lastoperationdate too"""
    accounting_management = AccountingManagement(connection)
    obj = accounting_management.get_operations()
    assert "getoperationsresult" in obj.content.tag
    obj = accounting_management.get_operations("21/12/2012-00:00:00")
    assert obj.content.request.lastoperationdate == "21/12/2012-00:00:00"
    testdate = date(2012, 12, 21)
    obj = accounting_management.get_operations(testdate)
    assert obj.content.request.lastoperationdate == "21/12/12-00:00:00"
    obj = None
    try:
        obj = accounting_management.get_operations("INVALIDDATE")
    except ShibaParameterError:
        pass
    assert obj is None


def test_get_compensation_details(connection):
    """get_compensation_details test, must fail"""
    accounting_management = AccountingManagement(connection)
    obj = None
    try:
        obj = accounting_management.get_compensation_details("1337")
    except ShibaParameterError:
        pass
    assert obj is None



# coding: utf-8
from __future__ import unicode_literals, print_function
import os
import pytest

from codecs import open
from contextlib import contextmanager

from requests import Response


def make_requests_get_mock(filename):
    def mockreturn(*args, **kwargs):
        response = Response()
        with open(os.path.join(os.path.dirname(__file__), 'Assets', filename), 'r', 'ISO-8859-1') as fd:
            response._content = fd.read().encode('utf-8')
        return response
    return mockreturn


def make_simple_text_mock(filename):
    def mockreturn(*args, **kwargs):
        with open(os.path.join(os.path.dirname(__file__), 'Assets', filename), 'r', 'ISO-8859-1') as fd:
            return fd.read()
    return mockreturn


@contextmanager
def assert_raises(exception_class, msg=None):
    """Check that an exception is raised and its message contains `msg`."""
    with pytest.raises(exception_class) as exception:
        yield
    if msg is not None:
        message = '%s' % exception
        assert msg.lower() in message.lower()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shiba.salesmanagement import SalesManagement
from shiba.shibaexceptions import *


def test_get_new_sales(connection):
    """regular get_new_sales test"""
    sales_management = SalesManagement(connection)
    obj = sales_management.get_new_sales()
    assert "getnewsalesresult" in obj.content.tag


def test_accept_sale(connection):
    """Only fail result, as accepting an actual sale is not simulable"""
    sales_management = SalesManagement(connection)
    itemid = "000000"
    obj = None
    try:
        obj = sales_management.accept_sale(itemid)
    except ShibaServiceError:
        pass
    except ShibaParameterError:
        pass


def test_refuse_sale(connection):
    """Only fail result, as refusing an actual sale is not simulable"""
    sales_management = SalesManagement(connection)
    itemid = "000000"
    obj = None
    try:
        obj = sales_management.refuse_sale(itemid)
    except ShibaServiceError:
        pass
    except ShibaParameterError:
        pass


def test_get_current_sales(connection):
    """get_current_sales test, on variable parameters, plus some fail results"""
    sales_management = SalesManagement(connection)
    obj = sales_management.get_current_sales()
    assert "getcurrentsalesresult" in obj.content.tag
    assert not obj.content.request.ispendingpreorder
    obj = sales_management.get_current_sales(ispendingpreorder="y")
    assert "getcurrentsalesresult" in obj.content.tag
    assert obj.content.request.ispendingpreorder
    try:
        sales_management.get_current_sales(ispendingpreorder="n")
    except ShibaCallingError:
        pass
    obj = sales_management.get_current_sales(purchasedate="WRONGDATE")
    for elem in obj.content.response:
        assert elem.tag != "purchasedate"
    obj = sales_management.get_current_sales(purchasedate="2012-12-21")
    assert "21/12/2012" == obj.content.request.purchasedate


def test_get_billing_information(connection):
    """get_billing_information test, will raise an error due to unknown purchaseid"""
    sales_management = SalesManagement(connection)
    obj = None
    try:
        obj = sales_management.get_billing_information("1337")
    except ShibaServiceError:
        pass
    assert obj is None


def test_get_shipping_information(connection):
    """get_billing_information test, will raise an error due to unknown purchaseid"""
    sales_management = SalesManagement(connection)
    obj = None
    try:
        obj = sales_management.get_shipping_information("1337")
    except ShibaServiceError:
        pass
    assert obj is None


def test_get_items_todo_list(connection):
    """get_items_todo_list routine test"""
    sales_management = SalesManagement(connection)
    obj = sales_management.get_item_todo_list()
    assert "getitemtodolistresult" in obj.content.tag


def test_get_item_infos(connection):
    """get_item_infos on a unknown product, must fail"""
    sales_management = SalesManagement(connection)
    obj = None
    try:
        obj = sales_management.get_item_infos("181063")
    except ShibaServiceError:
        pass
    assert obj is None


def test_cancel_item(connection):
    """cancel_item on an unknown product, must fail"""
    sales_management = SalesManagement(connection)
    obj = None
    try:
        obj = sales_management.cancel_item("1337", "comment")
    except ShibaServiceError:
        pass
    assert obj is None


def test_contact_us_about_item(connection):
    """contact_us_about_item on an unknown product, must fail"""
    sales_management = SalesManagement(connection)
    obj = None
    try:
        obj = sales_management.contact_us_about_item("1337", "message", "1337")
    except ShibaServiceError:
        pass
    assert obj is None


def test_contact_user_about_item(connection):
    """contact_user_about_item on an unknown product, must fail"""
    sales_management = SalesManagement(connection)
    obj = None
    try:
        obj = sales_management.contact_user_about_item("1337", "message")
    except ShibaServiceError:
        pass
    assert obj is None


def test_set_tracking_package_infos(connection):
    """set_tracking_package_infos on an unknown product, must fail. Testing internal error catching as well."""
    sales_management = SalesManagement(connection)
    obj = None
    try:
        obj = sales_management.set_tracking_package_infos("1337", "UPS", "0000000000")
    except ShibaParameterError:
        pass
    assert obj is None
    try:
        obj = sales_management.set_tracking_package_infos("1337", "Autre", "0000000000")
    except ShibaCallingError:
        pass
    assert obj is None


def test_confirm_preorder(connection):
    """confirm_preorder on an unknown advert, must fail. Testing internal error catching as well."""
    sales_management = SalesManagement(connection)
    obj = None
    try:
        obj = sales_management.confirm_preorder("1337", 1)
    except ShibaParameterError:
        pass
    assert obj is None
    try:
        obj = sales_management.confirm_preorder("1337", -8)
    except ShibaCallingError:
        pass
    assert obj is None


def test_wrong_user(fake_connection):
    sales_management = SalesManagement(fake_connection)
    try:
        sales_management.get_new_sales()
    except ShibaLoginError:
        pass



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shiba.shibaexceptions import ShibaRightsError
from shiba.inventorymanagement import InventoryManagement

import xmltodict
from lxml import objectify

import os


def test_product_types(connection):
    """product_types return test"""
    inventory_management = InventoryManagement(connection)
    ptypes = inventory_management.product_types()
    assert "producttypesresult" in ptypes.content.tag


def test_product_type_template(connection):
    """product_type_template tests on two scopes, for a fixed alias, plus a fail result"""
    inventory_management = InventoryManagement(connection)
    alias = "insolites_produit"
    ptemplate = inventory_management.product_type_template(alias, "")
    assert "producttypetemplateresult" in ptemplate.content.tag
    ptemplate = inventory_management.product_type_template(alias, "VALUES")
    assert "producttypetemplateresult" in ptemplate.content.tag


# @raises(ShibaParameterError)
# def test_product_type_template_fail(connection):
#     inventory_management = InventoryManagement(connection)
#     inventory_management.product_type_template("INVALIDALIAS", "INVALIDSCOPE")


def test_generic_import_file(connection):
    """generic_import_file test, from an XML file. Conversion is done by xmltodict from a dict or OrderedDict
    , as well with objectify with an objectified ElementTree element"""
    inventory_management = InventoryManagement(connection)

    f = open(os.path.dirname(os.path.realpath(__file__)) + "/Assets/genericimportfile.xml", "rb")
    testdict = xmltodict.parse(f)
    ret = inventory_management.generic_import_file(testdict)
    assert "OK" == ret.content.response.status

    f = open(os.path.dirname(os.path.realpath(__file__)) + "/Assets/genericimportfile.xml", "rb")
    testobj = objectify.parse(f)
    ret = inventory_management.generic_import_file(testobj)
    assert "OK" == ret.content.response.status


def test_generic_import_report(connection):
    """genreic_import_report method test from an import file call"""
    inventory_management = InventoryManagement(connection)
    f = open(os.path.dirname(os.path.realpath(__file__)) + "/Assets/genericimportfile.xml", "rb")
    testobj = objectify.parse(f)
    ret = inventory_management.generic_import_file(testobj)
    importid = ret.content.response.importid
    ret = inventory_management.generic_import_report(importid)
    assert "file" == ret.content.response.file.filename


def test_get_available_shipping_types(connection):
    inventory_management = InventoryManagement(connection)
    try:
        inventory_management.get_available_shipping_types()
    except ShibaRightsError:
        pass


# def test_export_inventory(connection):
#     """
#     This test fails if you do not use a PRO ACCOUNT.
#     :return:
#     """
#     inventory_management = InventoryManagement(connection)
#     print("This test fails if you do not use a PRO ACCOUNT.")
#     obj = inventory_management.export_inventory()
#     assert "inventoryresult" in obj.content.tag



# coding: utf-8
from __future__ import unicode_literals, print_function

import os

import pytest

from shiba.shibaconnection import ShibaConnection


ONLINE_TEST_ENABLED = os.environ.get("SHIBA_API_LOGIN", None) is not None


def get_connection():
    login = os.environ['SHIBA_API_LOGIN']
    password = os.environ['SHIBA_API_PASSWORD']
    return ShibaConnection(login, password, "https://ws.sandbox.priceminister.com")


def check_sandbox_up():
    if ONLINE_TEST_ENABLED:
        # connection = get_connection()
        # TODO: handle HTTP 50x error, to skip online test
        # import pdb; pdb.set_trace()  # noqa
        return True


@pytest.fixture(params=[pytest.mark.skipif(not check_sandbox_up(),  reason='need access to the sandbox')('parameter')])
def connection():
    return get_connection()


@pytest.fixture
def fake_connection():
    return ShibaConnection("Qvuik5bZVHbmg4mWsdsd0ik9", "RInDsddsdsdZ72tfA", "https://ws.sandbox.priceminister.com")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shiba.shibatools import inf_constructor, retrieve_obj_from_url, url_constructor, post_request
from shiba.shibaconnection import ShibaConnection


def test_retrieve_obj_from_url():
    """retrieve_obj_from_url test with a remote XML file"""
    obj = retrieve_obj_from_url("http://www.w3schools.com/xml/note.xml")
    assert "note" in obj.content.tag
    assert "to" in obj.content.to.tag and obj.content.to == "Tove"
    assert "heading" in obj.content.heading.tag and obj.content.heading == "Reminder"
    assert "body" in obj.content.body.tag and obj.content.body == "Don't forget me this weekend!"


def test_post_request():
    """
    Testing a post request with retrieve_obj_from_url
    This test method is dependant of the service used.
    (i.e., the content of the assert would change if the test service changes).
    http://httpbin.org/post returns the POST data you send.
    """
    post_data = "Who do you think you are to give me advice about dating?"
    ret = post_request("http://httpbin.org/post", post_data)
    assert post_data in ret


def test_inf_constructor():
    connection = ShibaConnection("test", "test")
    action = "genericimportreport"
    ret = inf_constructor(connection, action, inf1="info1", inf2="info2")
    assert "inf1" in ret
    assert "inf2" in ret
    assert ret["inf1"] == "info1"
    assert ret["action"] == "genericimportreport"


def test_url_constructor(fake_connection):
    connection = ShibaConnection("test", "test")
    action = "genericimportreport"
    ret = inf_constructor(connection, action, inf1="info1", inf2="info2")
    url = url_constructor(connection, ret)
    assert (url == "https://ws.priceminister.com/stock_ws?action=genericimportreport&inf1=info1&inf2=info2&login"
                   "=test&pwd=test&version=2011-11-29")



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from datetime import date

from .shibaconnection import ShibaConnection
from .shibatools import inf_constructor, url_constructor, retrieve_obj_from_url

from .compat import basestring


class AccountingManagement(object):
    """ Accounting Management class, showing global financial operations on your account, or specific financial details
        about an operation
    """

    def __init__(self, connection):
        if not isinstance(connection, ShibaConnection):
            raise ValueError("expecting a ShibaConnection instance, got '%s'" % type(connection))
        self.connection = connection

    def get_operations(self, lastoperationdate=""):
        """Get global operations which happened on your wallet, compensationid given back from XML can be used
        in the get_compensation_details method below to get more detailed information about a specific operation.

        :param lastoperationdate: as follows : dd/mm/yyyy-hh:mm:ss and as string or date instance.
        """
        operationcause = "salestransfer"

        if not isinstance(lastoperationdate, date) and not isinstance(lastoperationdate, basestring):
            raise ValueError("expected string or date for 'lastoperationdate', got '%s'" % type(lastoperationdate))

        if isinstance(lastoperationdate, date):
            lastoperationdate = lastoperationdate.strftime("%d/%m/%y-%H:%M:%S")
        inf = inf_constructor(self.connection, "getoperations", **locals())
        url = url_constructor(self.connection, inf)
        obj = retrieve_obj_from_url(url)
        return obj

    def get_compensation_details(self, compensationid):
        """Get a specific operation details from its "compensationid" found in the get_operation request return."""
        inf = inf_constructor(self.connection, "getcompensationdetails", **locals())
        url = url_constructor(self.connection, inf)
        obj = retrieve_obj_from_url(url)
        return obj



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shiba.marketplacemanagement import MarketplaceManagement
from shiba.shibaconnection import ShibaConnection

# from shiba.shibaexceptions import ShibaParameterError
# from . import assert_raises
from . import make_requests_get_mock


def test_get_product_list(monkeypatch):
    """testing get_product_list methods with different queries, with some invalid ones as well"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getproductlist.xml'))

    shiba_connection = ShibaConnection("test", "test" "https://ws.sandbox.priceminister.com")
    marketplace_management = MarketplaceManagement(shiba_connection)

    # TODO: to remove ?
    # with assert_raises(ShibaParameterError):
    #     marketplace_management.get_product_list()

    product_list = marketplace_management.get_product_list(kw="livre")
    assert "listingresult" in product_list.content.tag

    # TODO: to remove ?
    # with assert_raises(ShibaParameterError):
    #     marketplace_management.get_product_list(nbproductsperpage=-15, kw="livre")

    product_list = marketplace_management.get_product_list(kw="informatique", scope="PRICING")
    assert "listingresult" in product_list.content.tag


def test_get_category_map(monkeypatch):
    """get_category_map regular test"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getcategorymap.xml'))
    shiba_connection = ShibaConnection("test", "test" "https://ws.sandbox.priceminister.com")
    marketplace_management = MarketplaceManagement(shiba_connection)

    obj = marketplace_management.get_category_map()
    assert "categorymap" in obj.content.tag



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os.path as op
from shiba.inventorymanagement import InventoryManagement
from shiba.shibaconnection import ShibaConnection

import xmltodict
from lxml import objectify

from . import make_requests_get_mock, make_simple_text_mock


def test_product_types(monkeypatch):
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getproducttypes.xml'))
    inventory_management = InventoryManagement(ShibaConnection("test", "test", sandbox=True))
    ptypes = inventory_management.product_types()
    assert "producttypesresult" in ptypes.content.tag


def test_product_type_template(monkeypatch):
    """product_type_template tests on two scopes, for a fixed alias, plus a fail result"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getproducttypetemplate.xml'))
    alias = "insolites_produit"
    inventory_management = InventoryManagement(ShibaConnection("test", "test", sandbox=True))
    ptemplate = inventory_management.product_type_template(alias, "")
    assert "producttypetemplateresult" in ptemplate.content.tag


def test_get_available_shipping_types(monkeypatch):
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getavailableshippingtypes.xml'))
    inventory_management = InventoryManagement(ShibaConnection("test", "test", sandbox=True))
    obj = inventory_management.get_available_shipping_types()
    assert "getavailableshippingtypesresult" in obj.content.tag


def test_export_inventory(monkeypatch):
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_exportinventory.xml'))
    inventory_management = InventoryManagement(ShibaConnection("test", "test", sandbox=True))
    obj = inventory_management.export_inventory()
    assert "inventoryresult" in obj.content.tag


def test_generic_import_file(monkeypatch):
    """generic_import_file test, from an XML file. Conversion is done by xmltodict from a dict or OrderedDict , as well
    with objectify with an objectified ElementTree element"""

    monkeypatch.setattr('shiba.shibatools.post_request', make_simple_text_mock('sample_genericimportfile.xml'))

    f = open(op.join(op.dirname(__file__), 'Assets', 'genericimportfile.xml'), 'rb')
    testdict = xmltodict.parse(f)
    inventory_management = InventoryManagement(ShibaConnection("test", "test", sandbox=True))
    ret = inventory_management.generic_import_file(testdict)
    assert "OK" == ret.content.response.status

    f = open(op.join(op.dirname(__file__), 'Assets', 'genericimportfile.xml'), 'rb')
    testobj = objectify.parse(f)
    inventory_management = InventoryManagement(ShibaConnection("test", "test", sandbox=True))
    ret = inventory_management.generic_import_file(testobj)
    assert "OK" == ret.content.response.status



# -*- coding: utf-8 -*-
""" Tools used by Shiba data retrieving classes"""
from __future__ import unicode_literals

import re
from collections import OrderedDict

import requests
import xmltodict

from lxml import etree
from lxml import objectify

from .shibaconnection import ShibaConnection
from .shibaresponseobject import ShibaResponseObject
from .shibaexceptions import (ShibaParameterError, ShibaLoginError, ShibaQuotaExceededError,
                              ShibaRightsError, ShibaConnectionError, ShibaUnknownServiceError, ShibaServiceError,
                              ShibaCallingError)
from .compat import is_py3, to_unicode

if is_py3:
    from http.client import HTTPException
    from urllib.parse import urlencode
else:
    from urllib import urlencode
    from httplib import HTTPException


def post_request(url, data):
    """Method creating and submitting the multipart request of the wished to be imported XML file.

    :param data: raw XML as text

    :rtype: plain text from servers response
    """
    header = {"User-Agent": "Mozilla/5.0 (Windows; U; Windows NT 6.1; de-DE; rv:1.9.0.10) "
                            "Gecko/2009042316 Firefox/3.0.10 (.NET CLR 4.0.20506)"}
    data = to_unicode(data).encode('utf-8')
    d = {"file": data}
    r = requests.post(url, files=d, headers=header)
    return r.text


def retrieve_obj_from_url(url, data=None):
    """Give it an URL, will send you back an object based on received XML from WebService, it removes and store
    namespace as its not really useful for common usages of this API.
    You can also give it a dict or an objectified tree to POST a file (used for import webservice)
    :param url: fully formatted URL from url_constructor
    :param data: if not None, will send a POST request to URL instead of a GET one, with multipart sending of
    this parameter content

    :rtype : ShibaResponseObject object, containing lxml.objectify class, raw XML as Unicode string and the
    namespace from the XML.
    """
    try:
        if data is not None:
            xml = post_request(url, data)
        else:
            xml = requests.get(url).text
    except requests.ConnectionError:
        raise ShibaConnectionError("HTTP error = Connection error - On URL: " + url)
    except requests.HTTPError as e:
        raise ShibaConnectionError("URL error = " + to_unicode(e.reason) + " - On URL: " + url)
    except HTTPException:
        raise ShibaConnectionError("HTTP unknown error =" + " - On URL: " + url)
    except:
        raise
    try:
        namespace = re.search(pattern='xmlns="[^"]', string=xml)
        if namespace is not None:
            namespace = namespace.group()
        else:
            namespace = ""
        xmlepured = re.sub(pattern=' xmlns="[^"]+"', repl='', string=xml, flags=0)
        xmlepured = xmlepured.encode('utf-8')
        obj = objectify.fromstring(xmlepured)
    except:
        raise ShibaUnknownServiceError(
            "Unknown error from service or from internal modules : Service returned : " + xml)
    if _check_errors(obj) is not False:
        try:
            if "Unknown error" == obj.error.code:
                raise ShibaServiceError("Unknown error from WebService (maybe the sale isn't confirmed yet?)"
                                        " : " + obj.error.message + " - Reason : " + obj.error.details.detail)
        except ShibaServiceError:
            raise ShibaServiceError("Unknown error from WebService (maybe the sale isn't confirmed yet?)"
                                    " : " + obj.error.message + " - Reason : " + obj.error.details.detail)
        except:
            raise ShibaUnknownServiceError("An unknown error from the WebService has occurred - XML dump : " +
                                           etree.tostring(obj))
    return ShibaResponseObject(namespace, obj, xml.encode('utf-8'))


def create_xml_from_item_obj(inv):
    """Generate XML from the "inv" parameter.

    :param inv: is an object or a dict hierarchized as the XML structure described \
    in the WebServices documentation. Take a look at the xmltodict or the lxml.objectify documentation too.

    :rtype: string output from objectified element or ElementTree element tostring() methods
    """
    if type(inv) is etree._ElementTree:
        return etree.tostring(inv)
    elif type(inv) is dict or type(inv) is OrderedDict:
        try:
            return xmltodict.unparse(inv)
        except:
            raise ShibaCallingError("error from dictionary to xml : can't create XML from dictionary,"
                                    " refers to xmltodict documentation"
                                    "(you don't need to add a root element 'items' to your items)")
    else:
        raise ShibaCallingError("error : bad input parameter given, "
                                "expecting dict, objectify object or ElementTree element")


def inf_constructor(shibaconnection, action, **kwargs):
    """Just a selective concatenation of args given and related static data from the given action.

    :param shibaconnection: ShibaConnection class instance
    :param action: string giving the action type for the WebService
    :param kwargs: params to concatenate

    :rtype epured and updated dict
    """
    if isinstance(shibaconnection, ShibaConnection) is False:
        raise ShibaCallingError("Internal parameter error : shibaconnection parameter is not "
                                "a ShibaConnection instance")
    if action not in shibaconnection.actionsinfo:
        raise ShibaCallingError("Internal parameter error : action parameter " +
                                action + " is unknown from the actions list")
    newkwargs = {}
    for each in kwargs:
        if kwargs[each] is not None and kwargs[each] != "":
            newkwargs[each] = to_unicode(kwargs[each])
    newkwargs.update(shibaconnection.actionsinfo[action])
    newkwargs["action"] = action
    return newkwargs


def url_constructor(shibaconnection, inf, domain=None):
    """URL constructor, formatting input and adding as many URL arguments as given.

    :param inf: info dict, from inf_constructor
    :param domain: force URL domain, useful for some actions needing to go with http instead of https

    :rtype: formatted URL string
    """
    if domain is None:
        domain = shibaconnection.domain
    pop = inf.pop("cat")
    if "self" in inf:
        inf.pop("self")
    primary = "%s/%s?" % (domain, pop)
    ordered_inf = OrderedDict()
    for k in sorted([key for key in inf]):
        ordered_inf[k] = inf[k]
    url = primary + urlencode(ordered_inf)
    return url


def _check_errors(obj):
    """Errors checking from the returned XML as object. Handling errors as internal exceptions raising

    :param obj: objectified element to check errors from

    :rtype: objectified element if errors have been found, False if the WebService hasn't encountered an error
    """
    if "errorresponse" in obj.tag:
        if "ServerError" == obj.error.code:
            raise ShibaParameterError("Parameter error : " + obj.error.message +
                                      " - Reason : " + obj.error.details.detail)
        if "ParameterError" == obj.error.code:
            raise ShibaParameterError("Parameter error : " + obj.error.message +
                                      " - Reason : " + obj.error.details.detail)
        if "InvalidUserConnection" == obj.error.code:
            raise ShibaLoginError("Invalid user connection : " + obj.error.message +
                                  " - Reason : " + obj.error.details.detail)
        if "InvalidUserRights" == obj.error.code:
            if "Quota exceeded" in obj.error.message.text:
                raise ShibaQuotaExceededError("Too many requests : " + obj.error.message +
                                              " - Reason : " + obj.error.details.detail)
            else:
                raise ShibaRightsError("Invalid user rights : " + obj.error.message +
                                       " - Reason : " + obj.error.details.detail)
        return obj
    return False



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from .shibaconnection import ShibaConnection
from .shibatools import inf_constructor, url_constructor, retrieve_obj_from_url
from .compat import basestring


class MarketplaceManagement(object):
    """ Marketplace informations retrieving, such as product lists and category mapping"""

    def __init__(self, connection):
        if not isinstance(connection, ShibaConnection):
            raise ValueError("expecting a ShibaConnection instance, got '%s'" % type(connection))
        self.connection = connection

    def get_product_list(self, scope="", kw="", nav="", refs="", productids="", nbproductsperpage="", pagenumber=""):
        """Prints a search list result from given parameters.

        :param scope: none (classic results), "PRICING" (classic results plus 10 best announces)
            or "LIMITED" (search in categories in which rapid put on sale is possible through WS)
        :param kw: research keyword
        :param nav: navigation category (url friendly ones, can be found on PriceMinister categories' URLs)
        :param refs: EAN, or ISBN, as a string, each value separated by a coma ','.
        :param productids: same as refs but as products ID.
        :param nbproductsperpage: products per page, default is 20.
        :param pagenumber: page number, default is 1.
        """
        if not isinstance(refs, basestring) and not isinstance(refs, list):
            raise ValueError("expected string or list for 'refs', got '%s'" % type(refs))

        if not isinstance(productids, basestring) and not isinstance(productids, list):
            raise ValueError("expected string or list for 'productids', got '%s'" % type(productids))

        if isinstance(refs, list):
            refs = ','.join(refs)
        if isinstance(productids, list):
            productids = ','.join(productids)
        if kw != u'':
            # TODO: and ???
            pass
        inf = inf_constructor(self.connection, "listing", **locals())
        url = url_constructor(self.connection, inf)
        obj = retrieve_obj_from_url(url)
        return obj

    def get_category_map(self):
        """Lists items categories from the PriceMinister platform"""
        inf = inf_constructor(self.connection, "categorymap", **locals())
        url = url_constructor(self.connection, inf, domain="http://ws.priceminister.com")
        obj = retrieve_obj_from_url(url)
        return obj



# coding: utf-8
from __future__ import unicode_literals, print_function

import pytest

from contextlib import contextmanager


@contextmanager
def assert_raises(exception_class, msg=None):
    """Check that an exception is raised and its message contains `msg`."""
    with pytest.raises(exception_class) as exception:
        yield
    if msg is not None:
        message = '%s' % exception
        assert msg.lower() in message.lower()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import os.path as op
import xmltodict

from shiba.shibatools import inf_constructor, retrieve_obj_from_url, url_constructor
from shiba.shibaexceptions import ShibaQuotaExceededError
from shiba.shibaconnection import ShibaConnection

from shiba.inventorymanagement import InventoryManagement

from . import make_requests_get_mock, make_simple_text_mock, assert_raises


def test_retrieve_obj_from_url(monkeypatch):
    """retrieve_obj_from_url test with a remote XML file"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('note.xml'))
    obj = retrieve_obj_from_url("http://www.w3schools.com/xml/note.xml")
    assert "note" in obj.content.tag
    assert "to" in obj.content.to.tag and obj.content.to == "Tove"
    assert "heading" in obj.content.heading.tag and obj.content.heading == "Reminder"
    assert "body" in obj.content.body.tag and obj.content.body == "Don't forget me this weekend!"


def test_inf_constructor():
    connection = ShibaConnection("test", "test")
    action = "genericimportreport"
    ret = inf_constructor(connection, action, inf1="info1", inf2="info2")
    assert "inf1" in ret
    assert "inf2" in ret
    assert ret["inf1"] == "info1"
    assert ret["action"] == "genericimportreport"


def test_url_constructor():
    connection = ShibaConnection("test", "test")
    action = "genericimportreport"
    ret = inf_constructor(connection, action, inf1="info1", inf2="info2")
    url = url_constructor(connection, ret)
    assert ("https://ws.priceminister.com/stock_ws?action=genericimportreport&inf1=info1&inf2=info2&login=test&"
            "pwd=test&version=2011-11-29" == url)


def test_assert_quota_exeeded(monkeypatch):
    """raises ShibaQuotaExceededError exception"""
    monkeypatch.setattr('shiba.shibatools.post_request', make_simple_text_mock('quota_exceeded_message.xml'))

    connection = ShibaConnection("test", "test")
    inventory = InventoryManagement(connection)
    f = open(op.join(op.dirname(__file__), 'Assets', 'genericimportfile.xml'), 'rb')
    testdict = xmltodict.parse(f)
    with assert_raises(ShibaQuotaExceededError):
        inventory.generic_import_file(data=testdict)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shiba.salesmanagement import SalesManagement
from shiba.shibaconnection import ShibaConnection
from shiba.shibaexceptions import ShibaServiceError, ShibaParameterError, ShibaCallingError

from . import make_requests_get_mock


def test_get_new_sales(monkeypatch):
    """regular get_new_sales test"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getnewsales.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))

    obj = sales_management.get_new_sales()
    assert "getnewsalesresult" in obj.content.tag


def test_accept_sale(monkeypatch):
    """Only fail result, as accepting an actual sale is not simulable"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_accept_sale.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    itemid = "000000"
    obj = None
    try:
        obj = sales_management.accept_sale(itemid)
    except ShibaServiceError:
        pass
    except ShibaParameterError:
        pass
    assert obj is not None


def test_refuse_sale(monkeypatch):
    """Only fail result, as refusing an actual sale is not simulable"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_accept_sale.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    itemid = "000000"
    obj = None
    try:
        obj = sales_management.refuse_sale(itemid)
    except ShibaServiceError:
        pass
    except ShibaParameterError:
        pass
    assert obj is not None


def test_get_current_sales(monkeypatch):
    """get_current_sales test, on variable parameters, plus some fail results"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getcurrentsales.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = sales_management.get_current_sales()
    assert "getcurrentsalesresult" in obj.content.tag
    assert not obj.content.request.ispendingpreorder
    try:
        sales_management.get_current_sales(ispendingpreorder="n")
    except ShibaCallingError:
        pass
    obj = sales_management.get_current_sales(purchasedate="WRONGDATE")
    for elem in obj.content.response:
        assert elem.tag != "purchasedate"


def test_get_billing_information(monkeypatch):
    """get_billing_information test, will raise an error due to unknown purchaseid"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getbillinginformation.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = sales_management.get_billing_information("1337")
    assert obj.content.tag == "getbillinginformationresult"


def test_get_shipping_information(monkeypatch):
    """get_billing_information test"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getshippinginformation.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = None
    obj = sales_management.get_shipping_information("1337")
    assert obj is not None
    assert obj.content.tag == "getshippinginformationresult"


def test_get_items_todo_list(monkeypatch):
    """get_items_todo_list routine test"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getitemtodolist.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = sales_management.get_item_todo_list()
    assert "getitemtodolistresult" in obj.content.tag


def test_get_item_infos(monkeypatch):
    """get_item_infos on a product"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getiteminfos.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = sales_management.get_item_infos("181063")
    assert obj.content.tag == "getiteminfosresult"


def test_cancel_item(monkeypatch):
    """cancel_item test"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_cancelitem.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = sales_management.cancel_item("1337", "comment")
    assert obj.content.tag == "cancelitemresult"


def test_contact_us_about_item(monkeypatch):
    """contact_us_about_item test"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_contactus.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = sales_management.contact_us_about_item("1337", "message", "1337")
    assert obj.content.tag == "contactusaboutitemresult"


def test_contact_user_about_item(monkeypatch):
    """contact_user_about_item on a product"""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_contactuser.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = sales_management.contact_user_about_item("1337", "message")
    assert obj.content.tag == "contactuseraboutitemresult"


def test_set_tracking_package_infos(monkeypatch):
    """set_tracking_package_infos on a product. Testing internal error catching as well."""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_settrackingpackageinfos.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = sales_management.set_tracking_package_infos("1337", "UPS", "0000000000")
    assert obj.content.tag == "setshippingpackageinfosresult"
    obj = None
    try:
        obj = sales_management.set_tracking_package_infos("1337", "Autre", "0000000000")
    except ShibaCallingError:
        pass
    assert obj is None


def test_confirm_preorder(monkeypatch):
    """confirm_preorder on an advert. Testing internal error catching as well."""
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_confirmpreorder.xml'))
    sales_management = SalesManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    obj = sales_management.confirm_preorder("1337", 1)
    assert obj.content.tag == "confirmpreorder"
    obj = None
    try:
        obj = sales_management.confirm_preorder("1337", -8)
    except ShibaCallingError:
        pass
    assert obj is None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
import os.path as op


from codecs import open
from setuptools import setup, find_packages


def read(fname):
    ''' Return the file content. '''
    here = op.abspath(op.dirname(__file__))
    with open(op.join(here, fname), 'r', 'utf-8') as fd:
        return fd.read()

readme = read('README.rst')
changelog = read('CHANGES.rst').replace('.. :changelog:', '')

requirements = [
    "requests",
    "xmltodict",
    "lxml",
]

version = ''
version = re.search(r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
                    read(op.join('shiba', '__init__.py')),
                    re.MULTILINE).group(1)

if not version:
    raise RuntimeError('Cannot find version information')


setup(
    name='Shiba',
    author="Maxime Boguta",
    author_email="maxime.boguta@epitech.eu",
    version=version,
    url='https://github.com/ShibaAPI/shiba',
    packages=find_packages(),
    install_requires=requirements,
    zip_safe=True,
    description="A Python API for PriceMinister WebServices",
    long_description=readme + '\n\n' + changelog,
    keywords=["api", "priceminister", "python", "webservices"],
    license='GPLv3',
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Intended Audience :: Developers',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: Implementation :: CPython',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)



# -*- coding: utf-8 -*-
""" Those exception classes will usually be raised after an error returned from the WebServices."""
from __future__ import unicode_literals


class ShibaException(Exception):
    """Main exception class, as you can catch the whole of Shiba exceptions from it"""
    pass


class ShibaParameterError(ShibaException):
    """Shiba parameter error, XML returned from the WebService told that one or more parameters are incorrect."""
    pass


class ShibaLoginError(ShibaException):
    """Shiba login error, XML returned from the WebService told that the login informations given are incorrect."""
    pass


class ShibaRightsError(ShibaException):
    """Shiba rights error, means XML returned the specified login IDs aren't authorized to access/modify such ressource
    from the asked WebService"""


class ShibaServiceError(ShibaException):
    """Shiba service error, WebService told that an error have been encountered, but may can't precise where"""
    pass


class ShibaUnknownServiceError(ShibaException):
    """Shiba unknown service error, WebService has caused/encoutered an error, but we can't know why (followed by a
    rough print of the XML retrieved"""
    pass


class ShibaConnectionError(ShibaException):
    """Shiba connection error, kind of an internal error: URL can't be reached, in case of bad URL formatting or
    internet connection failure (HTTP errors 404, 500...)."""
    pass


class ShibaCallingError(ShibaException):
    """Shiba internal error, for code relative errors. If such an exception is raised, look at given parameters to
    the API methods first as this is the major exception raising point for this type of exception"""
    pass


class ShibaQuotaExceededError(ShibaException):
    """Shiba throtlle error, you have sumbitted too many request of a type within a certain amount of time"""
    pass



#!/usr/bin/env python
from __future__ import unicode_literals

from shiba.marketplacemanagement import MarketplaceManagement
from shiba.shibaexceptions import *


def test_get_product_list(connection):
    """testing get_product_list methods with different queries, with some invalid ones as well"""
    marketplace_management = MarketplaceManagement(connection)
    try:
        obj = marketplace_management.get_product_list()
    except ShibaParameterError:
        pass
    obj = marketplace_management.get_product_list(kw="livre")
    assert "listingresult" in obj.content.tag
    try:
        obj = marketplace_management.get_product_list(nbproductsperpage=-15, kw="livre")
    except ShibaParameterError:
        pass
    obj = marketplace_management.get_product_list(kw="informatique", scope="PRICING")
    assert "listingresult" in obj.content.tag


def test_get_category_map(connection):
    """get_category_map regular test"""
    marketplace_management = MarketplaceManagement(connection)
    obj = marketplace_management.get_category_map()
    assert "categorymap" in obj.content.tag



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shiba.accountingmanagement import AccountingManagement
from shiba.shibaconnection import ShibaConnection

from . import make_requests_get_mock


def test_get_operations(monkeypatch):
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getoperations.xml'))
    account = AccountingManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    operations = account.get_operations()
    assert "getoperationsresult" in operations.content.tag
    assert operations.content.request.user == "vendeur"
    assert operations.content.request.operationcause == "salestransfer"


def test_get_compensation_details(monkeypatch):
    monkeypatch.setattr('requests.get', make_requests_get_mock('sample_getcompensationdetails.xml'))
    account = AccountingManagement(ShibaConnection("test", "test", "https://ws.sandbox.priceminister.com"))
    compensation_details = account.get_compensation_details("1337")
    assert compensation_details.content.tag == "getcompensationdetailsresult"
